package zabi.uniud.cggwrapper;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.prefs.BackingStoreException;
import java.util.prefs.Preferences;

public class Config {
	
	private static final File current = new File(".cggsettings");
	private static final Preferences prefs = Preferences.userRoot();
	private static final String nodeName = "CGG-Wrapper-Settings";
	
	public boolean useAA;
	public boolean useGL;
	public boolean splash;

	public static Config getConfig() {
		
		if (current.exists() && current.isFile()) {
			try (DataInputStream br = new DataInputStream(new FileInputStream(current))) {
				Config c = new Config();
				c.useAA = br.readBoolean();
				c.useGL = br.readBoolean();
				c.splash = br.readBoolean();
				System.out.println("Found config file");
				return c;
			} catch (IOException e) {
				System.err.println("Invalid config file");
			}
		}
		
		try {
			if (prefs.nodeExists(nodeName)) {
				System.out.println("Found config registry keys");
				Preferences p = prefs.node(nodeName);
				if (!p.getBoolean("disabled", false)) {
					Config c = new Config();
					c.useAA = p.getBoolean("useAA", true);
					c.useGL = p.getBoolean("useGL", true);
					c.splash = p.getBoolean("splash", false);
					return c;
				} else {
					System.out.println("Registry configs are disabled");
				}
			}
		} catch (BackingStoreException e) {
			System.err.println("Invalid registry entry");
		}
		return null;
	}
	
	
	@Override
	public String toString() {
		return String.format("AA:%sGL:%sSS:%s", useAA, useGL, splash);
	}
	
	public static void delete() {
		prefs.node(nodeName).putBoolean("disabled", true);
		current.delete();
		prefs.remove(nodeName);
		try {
			prefs.sync();
		} catch (BackingStoreException e) {
			System.err.println("Can't delete registry keys");
		}
	}
	
	public static void writeToRegistry(Config c) {
		try {
			Preferences p = prefs.node(nodeName);
			p.putBoolean("useAA", c.useAA);
			p.putBoolean("useGL", c.useGL);
			p.putBoolean("splash", c.splash);
			p.putBoolean("disabled", false);
			p.sync();
		} catch (Exception e) {
			System.err.println("Can't write config to registry");
		}
	}
	
	public static void writeToFile(Config c) {
		try (DataOutputStream dos = new DataOutputStream(new FileOutputStream(current))) {
			dos.writeBoolean(c.useAA);
			dos.writeBoolean(c.useGL);
			dos.writeBoolean(c.splash);
		} catch (Exception e) {
			System.err.println("Can't write config to file");
		}
	}
}
